@@ -34,6 +34,9 @@ pub type PolynomialLabel = String;
 pub trait PCUniversalParams: CanonicalSerialize + CanonicalDeserialize + Clone + Debug + ToBytes + FromBytes {
     /// Outputs the maximum degree supported by the committer key.
     fn max_degree(&self) -> usize;
+
+    /// Supported degree bounds
+    fn supported_degree_bounds(&self) -> &[usize];
 }
 
 /// Defines the minimal interface of committer keys for any polynomial
@@ -49,7 +52,7 @@ pub trait PCCommitterKey: CanonicalSerialize + CanonicalDeserialize + Clone + De
 
 /// Defines the minimal interface of verifier keys for any polynomial
 /// commitment scheme.
-pub trait PCVerifierKey: CanonicalSerialize + CanonicalDeserialize + Clone + Debug {
+pub trait PCVerifierKey: CanonicalSerialize + CanonicalDeserialize + Clone + Debug + ToBytes + FromBytes {
     /// Outputs the maximum degree supported by the universal parameters
     /// `Self` was derived from.
     fn max_degree(&self) -> usize;
@@ -58,13 +61,6 @@ pub trait PCVerifierKey: CanonicalSerialize + CanonicalDeserialize + Clone + Deb
     fn supported_degree(&self) -> usize;
 }
 
-/// Defines the minimal interface of prepared verifier keys for any polynomial
-/// commitment scheme.
-pub trait PCPreparedVerifierKey<Unprepared: PCVerifierKey> {
-    /// prepare
-    fn prepare(vk: &Unprepared) -> Self;
-}
-
 /// Defines the minimal interface of commitments for any polynomial
 /// commitment scheme.
 pub trait PCCommitment: CanonicalDeserialize + CanonicalSerialize + Clone + Debug + ToBytes {
@@ -78,16 +74,9 @@ pub trait PCCommitment: CanonicalDeserialize + CanonicalSerialize + Clone + Debu
     fn is_in_correct_subgroup_assuming_on_curve(&self) -> bool;
 }
 
-/// Defines the minimal interface of prepared commitments for any polynomial
-/// commitment scheme.
-pub trait PCPreparedCommitment<Unprepared: PCCommitment>: Clone {
-    /// Prepare
-    fn prepare(commitment: &Unprepared) -> Self;
-}
-
 /// Defines the minimal interface of commitment randomness for any polynomial
 /// commitment scheme.
-pub trait PCRandomness: CanonicalSerialize + CanonicalDeserialize + Clone {
+pub trait PCRandomness: CanonicalSerialize + CanonicalDeserialize + Clone + Eq {
     /// Outputs empty randomness that does not hide the commitment.
     fn empty() -> Self;
 
@@ -100,7 +89,15 @@ pub trait PCRandomness: CanonicalSerialize + CanonicalDeserialize + Clone {
 
 /// Defines the minimal interface of evaluation proofs for any polynomial
 /// commitment scheme.
-pub trait PCProof: CanonicalSerialize + CanonicalDeserialize + Clone + ToBytes {}
+pub trait PCProof: CanonicalSerialize + CanonicalDeserialize + Clone + ToBytes {
+    fn is_hiding(&self) -> bool;
+}
+
+impl<P: PCProof> PCProof for Vec<P> {
+    fn is_hiding(&self) -> bool {
+        self.iter().any(|p| p.is_hiding())
+    }
+}
 
 /// A polynomial along with information about its degree bound (if any), and the
 /// maximum number of queries that will be made to it. This latter number determines
@@ -169,7 +166,7 @@ impl<F: Field> LabeledPolynomial<F> {
 }
 
 /// A commitment along with information about its degree bound (if any).
-#[derive(Clone, CanonicalSerialize)]
+#[derive(Clone, Debug, CanonicalSerialize)]
 pub struct LabeledCommitment<C: PCCommitment> {
     label: PolynomialLabel,
     commitment: C,
