@@ -16,13 +16,10 @@
 
 #![cfg_attr(not(feature = "std"), no_std)]
 //! A crate for polynomial commitment schemes.
-#![deny(unused_import_braces, trivial_casts, bare_trait_objects)]
-#![deny(unused_qualifications, variant_size_differences, stable_features)]
-#![deny(non_shorthand_field_patterns, unused_attributes)]
-#![deny(renamed_and_removed_lints, unused_allocation, unused_comparisons)]
-#![deny(const_err, unused_must_use, unused_mut, private_in_public)]
-#![deny(unused_extern_crates, trivial_numeric_casts)]
 #![forbid(unsafe_code)]
+#![allow(clippy::module_inception)]
+#![allow(clippy::too_many_arguments)]
+#![allow(clippy::type_complexity)]
 
 #[macro_use]
 extern crate derivative;
@@ -31,8 +28,15 @@ extern crate derivative;
 extern crate snarkvm_profiler;
 
 pub use snarkvm_algorithms::fft::DensePolynomial as Polynomial;
-use snarkvm_fields::Field;
-use snarkvm_utilities::{error as error_fn, errors::SerializationError, serialize::*, FromBytes, ToBytes};
+use snarkvm_fields::{Field, PrimeField, ToConstraintField};
+use snarkvm_utilities::{
+    error as error_fn,
+    errors::SerializationError,
+    serialize::*,
+    FromBytes,
+    ToBytes,
+    ToMinimalBits,
+};
 
 use core::{fmt::Debug, sync::atomic::AtomicBool};
 use rand_core::RngCore;
@@ -72,6 +76,7 @@ pub use constraints::*;
 /// Errors pertaining to query sets.
 pub mod error;
 pub use error::*;
+use snarkvm_algorithms::Prepare;
 
 /// A random number generator that bypasses some limitations of the Rust borrow
 /// checker.
@@ -112,7 +117,9 @@ pub mod sonic_pc;
 /// `p` that have previously been committed to. Each element of a `QuerySet` is a `(label, query)`
 /// pair, where `label` is the label of a polynomial in `p`, and `query` is the field element
 /// that `p[label]` is to be queried at.
-pub type QuerySet<'a, F> = BTreeSet<(String, F)>;
+///
+/// Added the third field: the point name.
+pub type QuerySet<'a, T> = BTreeSet<(String, (String, T))>;
 
 /// `Evaluations` is the result of querying a set of labeled polynomials or equations
 /// `p` at a `QuerySet` `Q`. It maps each element of `Q` to the resulting evaluation.
@@ -121,21 +128,27 @@ pub type QuerySet<'a, F> = BTreeSet<(String, F)>;
 pub type Evaluations<'a, F> = BTreeMap<(String, F), F>;
 
 /// A proof of satisfaction of linear combinations.
-#[derive(Clone, Debug, CanonicalSerialize, CanonicalDeserialize)]
-pub struct BatchLCProof<F: Field, PC: PolynomialCommitment<F>> {
+#[derive(Clone, Debug, PartialEq, Eq, CanonicalSerialize, CanonicalDeserialize)]
+pub struct BatchLCProof<F: PrimeField, CF: PrimeField, PC: PolynomialCommitment<F, CF>> {
     /// Evaluation proof.
     pub proof: PC::BatchProof,
     /// Evaluations required to verify the proof.
     pub evaluations: Option<Vec<F>>,
 }
 
-impl<F: Field, PC: PolynomialCommitment<F>> FromBytes for BatchLCProof<F, PC> {
+impl<F: PrimeField, CF: PrimeField, PC: PolynomialCommitment<F, CF>> PCProof for BatchLCProof<F, CF, PC> {
+    fn is_hiding(&self) -> bool {
+        self.proof.is_hiding()
+    }
+}
+
+impl<F: PrimeField, CF: PrimeField, PC: PolynomialCommitment<F, CF>> FromBytes for BatchLCProof<F, CF, PC> {
     fn read_le<R: Read>(mut reader: R) -> io::Result<Self> {
         CanonicalDeserialize::deserialize(&mut reader).map_err(|_| error_fn("could not deserialize struct"))
     }
 }
 
-impl<F: Field, PC: PolynomialCommitment<F>> ToBytes for BatchLCProof<F, PC> {
+impl<F: PrimeField, CF: PrimeField, PC: PolynomialCommitment<F, CF>> ToBytes for BatchLCProof<F, CF, PC> {
     fn write_le<W: Write>(&self, mut writer: W) -> io::Result<()> {
         CanonicalSerialize::serialize(self, &mut writer).map_err(|_| error_fn("could not serialize struct"))
     }
@@ -145,32 +158,46 @@ impl<F: Field, PC: PolynomialCommitment<F>> ToBytes for BatchLCProof<F, PC> {
 /// a sender to commit to multiple polynomials and later provide a succinct proof
 /// of evaluation for the corresponding commitments at a query set `Q`, while
 /// enforcing per-polynomial degree bounds.
-pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
+pub trait PolynomialCommitment<F: PrimeField, CF: PrimeField>: Sized + Clone + Debug + PartialEq + Eq {
     /// The universal parameters for the commitment scheme. These are "trimmed"
     /// down to `Self::CommitterKey` and `Self::VerifierKey` by `Self::trim`.
     type UniversalParams: PCUniversalParams + Clone;
     /// The committer key for the scheme; used to commit to a polynomial and then
     /// open the commitment to produce an evaluation proof.
-    type CommitterKey: PCCommitterKey + Clone;
+    type CommitterKey: PCCommitterKey + ToBytes + FromBytes + Clone + Send + Sync;
     /// The verifier key for the scheme; used to check an evaluation proof.
-    type VerifierKey: PCVerifierKey + Clone;
+    type VerifierKey: PCVerifierKey + Prepare<Self::PreparedVerifierKey> + ToConstraintField<CF> + Clone + Send + Sync;
     /// The prepared verifier key for the scheme; used to check an evaluation proof.
-    type PreparedVerifierKey: PCPreparedVerifierKey<Self::VerifierKey> + Clone;
+    type PreparedVerifierKey: Clone;
     /// The commitment to a polynomial.
-    type Commitment: PCCommitment + Clone;
+    type Commitment: PCCommitment
+        + Prepare<Self::PreparedCommitment>
+        + ToConstraintField<CF>
+        + ToMinimalBits
+        + Clone
+        + Debug
+        + PartialEq
+        + Eq
+        + Send
+        + Sync;
     /// The prepared commitment to a polynomial.
-    type PreparedCommitment: PCPreparedCommitment<Self::Commitment>;
+    type PreparedCommitment: Clone;
     /// The commitment randomness.
-    type Randomness: PCRandomness + Clone;
+    type Randomness: PCRandomness + Clone + Send + Sync;
     /// The evaluation proof for a single point.
-    type Proof: PCProof + Clone;
+    type Proof: PCProof + Clone + Sync + Send;
     /// The evaluation proof for a query set.
     type BatchProof: CanonicalSerialize
         + CanonicalDeserialize
         + Clone
+        + PCProof
         + From<Vec<Self::Proof>>
         + Into<Vec<Self::Proof>>
-        + Debug;
+        + PartialEq
+        + Eq
+        + Debug
+        + Send
+        + Sync;
 
     /// Constructs public parameters when given as input the maximum degree `degree`
     /// for the polynomial commitment scheme.
@@ -236,13 +263,12 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         query_set: &QuerySet<F>,
         opening_challenge: F,
         rands: impl IntoIterator<Item = &'a Self::Randomness>,
-        rng: Option<&mut dyn RngCore>,
+        _rng: Option<&mut dyn RngCore>,
     ) -> Result<Self::BatchProof, Error>
     where
         Self::Randomness: 'a,
         Self::Commitment: 'a,
     {
-        let rng = &mut crate::optional_rng::OptionalRng(rng);
         let poly_rand_comm: BTreeMap<_, _> = labeled_polynomials
             .into_iter()
             .zip(rands)
@@ -258,13 +284,15 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
 
         let mut query_to_labels_map = BTreeMap::new();
 
-        for (label, point) in query_set.iter() {
-            let labels = query_to_labels_map.entry(point).or_insert_with(BTreeSet::new);
-            labels.insert(label);
+        for (label, (point_name, point)) in query_set.iter() {
+            let labels = query_to_labels_map
+                .entry(point_name)
+                .or_insert((point, BTreeSet::new()));
+            labels.1.insert(label);
         }
 
-        let mut proofs = Vec::with_capacity(query_to_labels_map.len());
-        for (query, labels) in query_to_labels_map.into_iter() {
+        let mut pool = snarkvm_utilities::ExecutionPool::<Result<_, _>>::with_capacity(query_to_labels_map.len());
+        for (_point_name, (&query, labels)) in query_to_labels_map.into_iter() {
             let mut query_polys = Vec::with_capacity(labels.len());
             let mut query_rands = Vec::with_capacity(labels.len());
             let mut query_comms = Vec::with_capacity(labels.len());
@@ -279,22 +307,24 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
                 query_comms.push(*comm);
             }
 
-            let proof_time = start_timer!(|| "Creating proof");
-            let proof = Self::open(
-                ck,
-                query_polys,
-                query_comms,
-                *query,
-                opening_challenge,
-                query_rands,
-                Some(rng),
-            )?;
-
-            end_timer!(proof_time);
-
-            proofs.push(proof);
+            pool.add_job(move || {
+                let proof_time = start_timer!(|| "Creating proof");
+                let proof = Self::open(
+                    ck,
+                    query_polys,
+                    query_comms,
+                    query,
+                    opening_challenge,
+                    query_rands,
+                    None,
+                );
+                end_timer!(proof_time);
+                proof
+            });
         }
         end_timer!(open_time);
+        let proofs: Vec<_> = pool.execute_all();
+        let proofs = proofs.into_iter().collect::<Result<Vec<_>, _>>()?;
 
         Ok(proofs.into())
     }
@@ -329,9 +359,11 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
     {
         let commitments: BTreeMap<_, _> = commitments.into_iter().map(|c| (c.label(), c)).collect();
         let mut query_to_labels_map = BTreeMap::new();
-        for (label, point) in query_set.iter() {
-            let labels = query_to_labels_map.entry(point).or_insert_with(BTreeSet::new);
-            labels.insert(label);
+        for (label, (point_name, point)) in query_set.iter() {
+            let labels = query_to_labels_map
+                .entry(point_name)
+                .or_insert((point, BTreeSet::new()));
+            labels.1.insert(label);
         }
 
         // Implicit assumption: proofs are order in same manner as queries in
@@ -340,7 +372,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         assert_eq!(proofs.len(), query_to_labels_map.len());
 
         let mut result = true;
-        for ((query, labels), proof) in query_to_labels_map.into_iter().zip(proofs) {
+        for ((_point_name, (query, labels)), proof) in query_to_labels_map.into_iter().zip(proofs) {
             let mut comms: Vec<&'_ LabeledCommitment<_>> = Vec::with_capacity(labels.len());
             let mut values = Vec::with_capacity(labels.len());
             for label in labels.into_iter() {
@@ -378,7 +410,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         opening_challenge: F,
         rands: impl IntoIterator<Item = &'a Self::Randomness>,
         rng: Option<&mut dyn RngCore>,
-    ) -> Result<BatchLCProof<F, Self>, Error>
+    ) -> Result<BatchLCProof<F, CF, Self>, Error>
     where
         Self::Randomness: 'a,
         Self::Commitment: 'a,
@@ -411,7 +443,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         commitments: impl IntoIterator<Item = &'a LabeledCommitment<Self::Commitment>>,
         eqn_query_set: &QuerySet<F>,
         eqn_evaluations: &Evaluations<F>,
-        proof: &BatchLCProof<F, Self>,
+        proof: &BatchLCProof<F, CF, Self>,
         opening_challenge: F,
         rng: &mut R,
     ) -> Result<bool, Error>
@@ -426,9 +458,14 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         let lc_s: BTreeMap<_, _> = linear_combinations.into_iter().map(|lc| (lc.label(), lc)).collect();
 
         let poly_query_set = lc_query_set_to_poly_query_set(lc_s.values().copied(), eqn_query_set);
-        let poly_evals: Evaluations<_> = poly_query_set.iter().cloned().zip(evals.clone().unwrap()).collect();
+        let poly_evals: Evaluations<_> = poly_query_set
+            .iter()
+            .map(|(_, point)| point)
+            .cloned()
+            .zip(evals.clone().unwrap())
+            .collect();
 
-        for &(ref lc_label, point) in eqn_query_set {
+        for &(ref lc_label, (_, point)) in eqn_query_set {
             if let Some(lc) = lc_s.get(lc_label) {
                 let claimed_rhs = *eqn_evaluations
                     .get(&(lc_label.clone(), point))
@@ -483,7 +520,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         query_set: &QuerySet<F>,
         opening_challenges: &dyn Fn(u64) -> F,
         rands: impl IntoIterator<Item = &'a Self::Randomness>,
-    ) -> Result<BatchLCProof<F, Self>, Error>
+    ) -> Result<BatchLCProof<F, CF, Self>, Error>
     where
         Self::Randomness: 'a,
         Self::Commitment: 'a;
@@ -495,7 +532,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         commitments: impl IntoIterator<Item = &'a LabeledCommitment<Self::Commitment>>,
         eqn_query_set: &QuerySet<F>,
         eqn_evaluations: &Evaluations<F>,
-        proof: &BatchLCProof<F, Self>,
+        proof: &BatchLCProof<F, CF, Self>,
         opening_challenges: &dyn Fn(u64) -> F,
         rng: &mut R,
     ) -> Result<bool, Error>
@@ -510,7 +547,7 @@ pub fn evaluate_query_set<'a, F: Field>(
 ) -> Evaluations<'a, F> {
     let polys: BTreeMap<_, _> = polys.into_iter().map(|p| (p.label(), p)).collect();
     let mut evaluations = Evaluations::new();
-    for (label, point) in query_set {
+    for (label, (_point_name, point)) in query_set {
         let poly = polys.get(label).expect("polynomial in evaluated lc is not found");
         let eval = poly.evaluate(*point);
         evaluations.insert((label.clone(), *point), eval);
@@ -525,11 +562,11 @@ fn lc_query_set_to_poly_query_set<'a, F: 'a + Field>(
     let mut poly_query_set = QuerySet::new();
     let lc_s = linear_combinations.into_iter().map(|lc| (lc.label(), lc));
     let linear_combinations: BTreeMap<_, _> = lc_s.collect();
-    for (lc_label, point) in query_set {
+    for (lc_label, (point_name, point)) in query_set {
         if let Some(lc) = linear_combinations.get(lc_label) {
             for (_, poly_label) in lc.iter().filter(|(_, l)| !l.is_one()) {
                 if let LCTerm::PolyLabel(l) = poly_label {
-                    poly_query_set.insert((l.into(), *point));
+                    poly_query_set.insert((l.into(), (point_name.clone(), *point)));
                 }
             }
         }
@@ -541,7 +578,6 @@ fn lc_query_set_to_poly_query_set<'a, F: 'a + Field>(
 pub mod tests {
     use crate::*;
     use rand::{distributions::Distribution, Rng};
-    use snarkvm_fields::Field;
     use snarkvm_utilities::rand::test_rng;
 
     #[derive(Default)]
@@ -555,21 +591,22 @@ pub mod tests {
         num_equations: Option<usize>,
     }
 
-    pub struct TestComponents<F: Field, PC: PolynomialCommitment<F>> {
+    pub struct TestComponents<F: PrimeField, CF: PrimeField, PC: PolynomialCommitment<F, CF>> {
         pub verification_key: PC::VerifierKey,
         pub commitments: Vec<LabeledCommitment<PC::Commitment>>,
         pub query_set: QuerySet<'static, F>,
         pub evaluations: Evaluations<'static, F>,
-        pub batch_lc_proof: Option<BatchLCProof<F, PC>>,
+        pub batch_lc_proof: Option<BatchLCProof<F, CF, PC>>,
         pub batch_proof: Option<PC::BatchProof>,
         pub opening_challenge: F,
         pub randomness: Vec<PC::Randomness>,
     }
 
-    pub fn bad_degree_bound_test<F, PC>() -> Result<(), Error>
+    pub fn bad_degree_bound_test<F, CF, PC>() -> Result<(), Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let rng = &mut test_rng();
         let max_degree = 100;
@@ -605,7 +642,7 @@ pub mod tests {
             let mut values = Evaluations::new();
             let point = F::rand(rng);
             for (i, label) in labels.iter().enumerate() {
-                query_set.insert((label.clone(), point));
+                query_set.insert((label.clone(), ("rand".into(), point)));
                 let value = polynomials[i].evaluate(point);
                 values.insert((label.clone(), point), value);
             }
@@ -627,10 +664,11 @@ pub mod tests {
         Ok(())
     }
 
-    fn test_template<F, PC>(info: TestInfo) -> Result<Vec<TestComponents<F, PC>>, Error>
+    fn test_template<F, CF, PC>(info: TestInfo) -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let TestInfo {
             num_iters,
@@ -645,12 +683,13 @@ pub mod tests {
         let mut test_components = Vec::new();
 
         let rng = &mut test_rng();
-        let max_degree = max_degree.unwrap_or_else(|| rand::distributions::Uniform::from(2..=64).sample(rng));
+        let max_degree = max_degree.unwrap_or_else(|| rand::distributions::Uniform::from(8..=64).sample(rng));
         let pp = PC::setup(max_degree, rng)?;
+        let supported_degree_bounds = pp.supported_degree_bounds();
 
         for _ in 0..num_iters {
             let supported_degree =
-                supported_degree.unwrap_or_else(|| rand::distributions::Uniform::from(1..=max_degree).sample(rng));
+                supported_degree.unwrap_or_else(|| rand::distributions::Uniform::from(4..=max_degree).sample(rng));
             assert!(max_degree >= supported_degree, "max_degree < supported_degree");
             let mut polynomials = Vec::new();
             let mut degree_bounds = if enforce_degree_bounds { Some(Vec::new()) } else { None };
@@ -666,11 +705,23 @@ pub mod tests {
                 let degree = rand::distributions::Uniform::from(1..=supported_degree).sample(rng);
                 let poly = Polynomial::rand(degree, rng);
 
+                let supported_degree_bounds_after_trimmed = supported_degree_bounds
+                    .iter()
+                    .copied()
+                    .filter(|x| *x >= degree && *x < supported_degree)
+                    .collect::<Vec<usize>>();
+
                 let degree_bound = if let Some(degree_bounds) = &mut degree_bounds {
-                    let range = rand::distributions::Uniform::from(degree..=supported_degree);
-                    let degree_bound = range.sample(rng);
-                    degree_bounds.push(degree_bound);
-                    Some(degree_bound)
+                    if !supported_degree_bounds_after_trimmed.is_empty() && rng.gen() {
+                        let range = rand::distributions::Uniform::from(0..supported_degree_bounds_after_trimmed.len());
+                        let idx = range.sample(rng);
+
+                        let degree_bound = supported_degree_bounds_after_trimmed[idx];
+                        degree_bounds.push(degree_bound);
+                        Some(degree_bound)
+                    } else {
+                        None
+                    }
                 } else {
                     None
                 };
@@ -701,11 +752,11 @@ pub mod tests {
             let mut query_set = QuerySet::new();
             let mut values = Evaluations::new();
             // let mut point = F::one();
-            for _ in 0..num_points_in_query_set {
+            for point_id in 0..num_points_in_query_set {
                 let point = F::rand(rng);
-                for (i, label) in labels.iter().enumerate() {
-                    query_set.insert((label.clone(), point));
-                    let value = polynomials[i].evaluate(point);
+                for (polynomial, label) in polynomials.iter().zip(labels.iter()) {
+                    query_set.insert((label.clone(), (format!("rand_{}", point_id), point)));
+                    let value = polynomial.evaluate(point);
                     values.insert((label.clone(), point), value);
                 }
             }
@@ -748,10 +799,11 @@ pub mod tests {
         Ok(test_components)
     }
 
-    fn equation_test_template<F, PC>(info: TestInfo) -> Result<Vec<TestComponents<F, PC>>, Error>
+    fn equation_test_template<F, CF, PC>(info: TestInfo) -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let TestInfo {
             num_iters,
@@ -766,12 +818,13 @@ pub mod tests {
         let mut test_components = Vec::new();
 
         let rng = &mut test_rng();
-        let max_degree = max_degree.unwrap_or_else(|| rand::distributions::Uniform::from(2..=64).sample(rng));
+        let max_degree = max_degree.unwrap_or_else(|| rand::distributions::Uniform::from(8..=64).sample(rng));
         let pp = PC::setup(max_degree, rng)?;
+        let supported_degree_bounds = pp.supported_degree_bounds();
 
         for _ in 0..num_iters {
             let supported_degree =
-                supported_degree.unwrap_or_else(|| rand::distributions::Uniform::from(1..=max_degree).sample(rng));
+                supported_degree.unwrap_or_else(|| rand::distributions::Uniform::from(4..=max_degree).sample(rng));
             assert!(max_degree >= supported_degree, "max_degree < supported_degree");
             let mut polynomials = Vec::new();
             let mut degree_bounds = if enforce_degree_bounds { Some(Vec::new()) } else { None };
@@ -787,10 +840,18 @@ pub mod tests {
                 let degree = rand::distributions::Uniform::from(1..=supported_degree).sample(rng);
                 let poly = Polynomial::rand(degree, rng);
 
+                let supported_degree_bounds_after_trimmed = supported_degree_bounds
+                    .iter()
+                    .copied()
+                    .filter(|x| *x >= degree && *x < supported_degree)
+                    .collect::<Vec<usize>>();
+
                 let degree_bound = if let Some(degree_bounds) = &mut degree_bounds {
-                    if rng.gen() {
-                        let range = rand::distributions::Uniform::from(degree..=supported_degree);
-                        let degree_bound = range.sample(rng);
+                    if !supported_degree_bounds_after_trimmed.is_empty() && rng.gen() {
+                        let range = rand::distributions::Uniform::from(0..supported_degree_bounds_after_trimmed.len());
+                        let idx = range.sample(rng);
+
+                        let degree_bound = supported_degree_bounds_after_trimmed[idx];
                         degree_bounds.push(degree_bound);
                         Some(degree_bound)
                     } else {
@@ -859,7 +920,7 @@ pub mod tests {
                     if !lc.is_empty() {
                         linear_combinations.push(lc);
                         // Insert query
-                        query_set.insert((label.clone(), point));
+                        query_set.insert((label.clone(), (format!("rand_{}", i), point)));
                     }
                 }
             }
@@ -917,10 +978,11 @@ pub mod tests {
         Ok(test_components)
     }
 
-    pub fn single_poly_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn single_poly_test<F, CF, PC>() -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -931,13 +993,14 @@ pub mod tests {
             max_num_queries: 1,
             ..Default::default()
         };
-        test_template::<F, PC>(info)
+        test_template::<F, CF, PC>(info)
     }
 
-    pub fn linear_poly_degree_bound_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn linear_poly_degree_bound_test<F, CF, PC>() -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -948,13 +1011,14 @@ pub mod tests {
             max_num_queries: 1,
             ..Default::default()
         };
-        test_template::<F, PC>(info)
+        test_template::<F, CF, PC>(info)
     }
 
-    pub fn single_poly_degree_bound_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn single_poly_degree_bound_test<F, CF, PC>() -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -965,13 +1029,15 @@ pub mod tests {
             max_num_queries: 1,
             ..Default::default()
         };
-        test_template::<F, PC>(info)
+        test_template::<F, CF, PC>(info)
     }
 
-    pub fn quadratic_poly_degree_bound_multiple_queries_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn quadratic_poly_degree_bound_multiple_queries_test<F, CF, PC>()
+    -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -982,13 +1048,14 @@ pub mod tests {
             max_num_queries: 2,
             ..Default::default()
         };
-        test_template::<F, PC>(info)
+        test_template::<F, CF, PC>(info)
     }
 
-    pub fn single_poly_degree_bound_multiple_queries_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn single_poly_degree_bound_multiple_queries_test<F, CF, PC>() -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -999,13 +1066,14 @@ pub mod tests {
             max_num_queries: 2,
             ..Default::default()
         };
-        test_template::<F, PC>(info)
+        test_template::<F, CF, PC>(info)
     }
 
-    pub fn two_polys_degree_bound_single_query_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn two_polys_degree_bound_single_query_test<F, CF, PC>() -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -1016,13 +1084,14 @@ pub mod tests {
             max_num_queries: 1,
             ..Default::default()
         };
-        test_template::<F, PC>(info)
+        test_template::<F, CF, PC>(info)
     }
 
-    pub fn full_end_to_end_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn full_end_to_end_test<F, CF, PC>() -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -1033,13 +1102,14 @@ pub mod tests {
             max_num_queries: 5,
             ..Default::default()
         };
-        test_template::<F, PC>(info)
+        test_template::<F, CF, PC>(info)
     }
 
-    pub fn full_end_to_end_equation_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn full_end_to_end_equation_test<F, CF, PC>() -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -1050,13 +1120,14 @@ pub mod tests {
             max_num_queries: 5,
             num_equations: Some(10),
         };
-        equation_test_template::<F, PC>(info)
+        equation_test_template::<F, CF, PC>(info)
     }
 
-    pub fn single_equation_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn single_equation_test<F, CF, PC>() -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -1067,13 +1138,14 @@ pub mod tests {
             max_num_queries: 1,
             num_equations: Some(1),
         };
-        equation_test_template::<F, PC>(info)
+        equation_test_template::<F, CF, PC>(info)
     }
 
-    pub fn two_equation_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn two_equation_test<F, CF, PC>() -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -1084,13 +1156,14 @@ pub mod tests {
             max_num_queries: 1,
             num_equations: Some(2),
         };
-        equation_test_template::<F, PC>(info)
+        equation_test_template::<F, CF, PC>(info)
     }
 
-    pub fn two_equation_degree_bound_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
+    pub fn two_equation_degree_bound_test<F, CF, PC>() -> Result<Vec<TestComponents<F, CF, PC>>, Error>
     where
-        F: Field,
-        PC: PolynomialCommitment<F>,
+        F: PrimeField,
+        CF: PrimeField,
+        PC: PolynomialCommitment<F, CF>,
     {
         let info = TestInfo {
             num_iters: 100,
@@ -1101,6 +1174,6 @@ pub mod tests {
             max_num_queries: 1,
             num_equations: Some(2),
         };
-        equation_test_template::<F, PC>(info)
+        equation_test_template::<F, CF, PC>(info)
     }
 }
