@@ -20,11 +20,12 @@ use snarkvm_curves::PairingEngine;
 use snarkvm_gadgets::{
     fields::FpGadget,
     traits::{alloc::AllocGadget, curves::PairingGadget},
+    PrepareGadget,
 };
-use snarkvm_r1cs::{ConstraintSystem, SynthesisError, ToConstraintField};
+use snarkvm_r1cs::{ConstraintSystem, SynthesisError};
 
 use crate::{
-    marlin_pc::{Commitment, CommitmentVar},
+    marlin_pc::{Commitment, CommitmentVar, PreparedLabeledCommitmentVar},
     LabeledCommitment,
     String,
     ToString,
@@ -35,10 +36,7 @@ pub struct LabeledCommitmentVar<
     TargetCurve: PairingEngine,
     BaseCurve: PairingEngine,
     PG: PairingGadget<TargetCurve, <BaseCurve as PairingEngine>::Fr>,
-> where
-    <TargetCurve as PairingEngine>::G1Affine: ToConstraintField<<BaseCurve as PairingEngine>::Fr>,
-    <TargetCurve as PairingEngine>::G2Affine: ToConstraintField<<BaseCurve as PairingEngine>::Fr>,
-{
+> {
     /// A text label for the commitment.
     pub label: String,
     /// The plain commitment.
@@ -52,8 +50,6 @@ where
     TargetCurve: PairingEngine,
     BaseCurve: PairingEngine,
     PG: PairingGadget<TargetCurve, <BaseCurve as PairingEngine>::Fr>,
-    <TargetCurve as PairingEngine>::G1Affine: ToConstraintField<<BaseCurve as PairingEngine>::Fr>,
-    <TargetCurve as PairingEngine>::G2Affine: ToConstraintField<<BaseCurve as PairingEngine>::Fr>,
 {
     fn clone(&self) -> Self {
         LabeledCommitmentVar {
@@ -71,8 +67,6 @@ where
     TargetCurve: PairingEngine,
     BaseCurve: PairingEngine,
     PG: PairingGadget<TargetCurve, <BaseCurve as PairingEngine>::Fr>,
-    <TargetCurve as PairingEngine>::G1Affine: ToConstraintField<<BaseCurve as PairingEngine>::Fr>,
-    <TargetCurve as PairingEngine>::G2Affine: ToConstraintField<<BaseCurve as PairingEngine>::Fr>,
 {
     fn alloc_constant<
         Fn: FnOnce() -> Result<T, SynthesisError>,
@@ -180,6 +174,26 @@ where
     }
 }
 
+impl<TargetCurve, BaseCurve, PG>
+    PrepareGadget<PreparedLabeledCommitmentVar<TargetCurve, BaseCurve, PG>, <BaseCurve as PairingEngine>::Fr>
+    for LabeledCommitmentVar<TargetCurve, BaseCurve, PG>
+where
+    TargetCurve: PairingEngine,
+    BaseCurve: PairingEngine,
+    PG: PairingGadget<TargetCurve, <BaseCurve as PairingEngine>::Fr>,
+{
+    fn prepare<CS: ConstraintSystem<<BaseCurve as PairingEngine>::Fr>>(
+        &self,
+        cs: CS,
+    ) -> Result<PreparedLabeledCommitmentVar<TargetCurve, BaseCurve, PG>, SynthesisError> {
+        Ok(PreparedLabeledCommitmentVar::<TargetCurve, BaseCurve, PG> {
+            label: self.label.clone(),
+            prepared_commitment: self.commitment.prepare(cs)?,
+            degree_bound: self.degree_bound.clone(),
+        })
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use snarkvm_algorithms::fft::DensePolynomial;
